@@ -1,6 +1,6 @@
 /* -*- c++ -*- */
 /*
- * Copyright 2003,2004 Free Software Foundation, Inc.
+ * Copyright 2003,2004,2006 Free Software Foundation, Inc.
  * 
  * This file is part of GNU Radio
  * 
@@ -207,4 +207,32 @@
 #define FR_USER_15	79
 
 
+// =======================================================================
+// READBACK Registers
+// =======================================================================
+
+#define FR_RB_IO_RX_A_IO_TX_A	1	// read back a-side i/o pins
+#define	FR_RB_IO_RX_B_IO_TX_B	2	// read back b-side i/o pins
+
+// ------------------------------------------------------------------------
+// FPGA Capability register
+//
+//    3                   2                   1                       
+//  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
+// +-----------------------------------------------+-+-----+-+-----+
+// |                    Reserved (Must be zero)    |T|NDUC |R|NDDC |
+// +-----------------------------------------------+-+-----+-+-----+
+//
+// Bottom 4-bits are Rx capabilities
+// Next   4-bits are Tx capabilities
+
+#define	FR_RB_CAPS	3
+#  define bmFR_RB_CAPS_NDDC_MASK	(0x7 << 0)   // # of digital down converters 0,1,2,4
+#  define bmFR_RB_CAPS_NDDC_SHIFT	0
+#  define bmFR_RB_CAPS_RX_HAS_HALFBAND	(0x1 << 3)
+#  define bmFR_RB_CAPS_NDUC_MASK        (0x7 << 4)   // # of digital up converters 0,1,2
+#  define bmFR_RB_CAPS_NDUC_SHIFT	4
+#  define bmFR_RB_CAPS_TX_HAS_HALFBAND	(0x1 << 7)
+
+
 #endif /* INCLUDED_FPGA_REGS_STANDARD_H */
