@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright 2004 Free Software Foundation, Inc.
+# Copyright 2004,2006 Free Software Foundation, Inc.
 # 
 # This file is part of GNU Radio
 # 
@@ -145,7 +145,7 @@ def build_shell_script (out, ihx_filename, rev):
     image = build_eeprom_image (ihx_filename, rev)
 
     out.write ('#!/bin/sh\n')
-    out.write ('usrper -x load_firmware /usr/local/share/usrp/rev%d/usrp_firmware.ihx\n' % rev)
+    out.write ('usrper -x load_firmware /usr/local/share/usrp/rev%d/std.ihx\n' % rev)
     out.write ('sleep 1\n')
     
     # print "len(image) =", len(image)
@@ -167,14 +167,14 @@ if __name__ == '__main__':
     usage = "usage: %prog -r REV [options] bootfile.ihx"
     parser = OptionParser (usage=usage)
     parser.add_option ("-r", "--rev", type="int", default=-1,
-                       help="Specify USRP revision number REV (1 or 2)")
+                       help="Specify USRP revision number REV (2 or 4)")
     (options, args) = parser.parse_args ()
     if len (args) != 1:
         parser.print_help ()
         sys.exit (1)
     if options.rev < 0:
         sys.stderr.write (
-            "You must specify the USRP revision number (0, 1 or 2) with -r REV\n")
+            "You must specify the USRP revision number (2 or 4) with -r REV\n")
         sys.exit (1)
 
     ihx_filename = args[0]
