@@ -24,11 +24,8 @@
 
 #include "fpga_load.h"
 
-#ifdef HAVE_USRP0
-#include "fpga_rev0.h"
-#endif
-#if defined(HAVE_USRP1) || defined(HAVE_USRP2)
-#include "fpga_rev1.h"
+#if defined(HAVE_USRP2)
+#include "fpga_rev2.h"
 #endif
 
 #endif /* INCLUDED_FPGA_H */
