@@ -23,8 +23,15 @@
 #include <fusb.h>
 #include <fusb_darwin.h>
 
+static const int MAX_BLOCK_SIZE = 16 * 1024;		// hard limit
+
 fusb_devhandle *
 fusb_sysconfig::make_devhandle (usb_dev_handle *udh)
 {
   return new fusb_devhandle_darwin (udh);
 }
+
+int fusb_sysconfig::max_block_size ()
+{
+  return MAX_BLOCK_SIZE;
+}
