@@ -1,6 +1,6 @@
 /* -*- c++ -*- */
 /*
- * Copyright 2003,2004 Free Software Foundation, Inc.
+ * Copyright 2003,2004,2006 Free Software Foundation, Inc.
  * 
  * This file is part of GNU Radio
  * 
@@ -28,6 +28,8 @@
 #include "usrp_commands.h"
 #include "usrp_ids.h"
 #include "usrp_i2c_addr.h"
+#include "fpga_regs_common.h"
+#include "fpga_regs_standard.h"
 #include <usb.h>
 #include <errno.h>
 #include <stdio.h>
@@ -56,8 +58,8 @@ static const int hash_slot_addr[2] = {
   USRP_HASH_SLOT_1_ADDR
 };
 
-static char *usrp_firmware_filename = "usrp_firmware.ihx";
-static char *usrp_fpga_filename = "usrp_fpga.rbf";
+static char *default_firmware_filename = "std.ihx";
+static char *default_fpga_filename     = "std_2rxhb_2tx.rbf";
 
 #include "std_paths.h"
 
@@ -84,6 +86,20 @@ find_file (const char *filename, int hw_rev)
   return 0;
 }
 
+static const char *
+get_proto_filename(const std::string user_filename, const char *env_var, const char *def)
+{
+  if (user_filename.length() != 0)
+    return user_filename.c_str();
+
+  char *s = getenv(env_var);
+  if (s && *s)
+    return s;
+
+  return def;
+}
+
+
 static void power_down_9862s (struct usb_dev_handle *udh);
 
 void
@@ -129,8 +145,8 @@ dev_handle_to_dev (usb_dev_handle *udh)
 /*
  * q must be a real USRP, not an FX2.  Return its hardware rev number.
  */
-static int
-_usrp_hw_rev (struct usb_device *q)
+int
+usrp_hw_rev (struct usb_device *q)
 {
   return q->descriptor.bcdDevice & 0x00FF;
 }
@@ -161,19 +177,19 @@ usrp_fx2_p (struct usb_device *q)
 bool
 usrp_usrp0_p (struct usb_device *q)
 {
-  return usrp_usrp_p (q) && _usrp_hw_rev (q) == 0;
+  return usrp_usrp_p (q) && usrp_hw_rev (q) == 0;
 }
 
 bool
 usrp_usrp1_p (struct usb_device *q)
 {
-  return usrp_usrp_p (q) && _usrp_hw_rev (q) == 1;
+  return usrp_usrp_p (q) && usrp_hw_rev (q) == 1;
 }
 
 bool
 usrp_usrp2_p (struct usb_device *q)
 {
-  return usrp_usrp_p (q) && _usrp_hw_rev (q) == 2;
+  return usrp_usrp_p (q) && usrp_hw_rev (q) == 2;
 }
 
 
@@ -228,7 +244,7 @@ usrp_open_interface (struct usb_device *dev, int interface, int altinterface)
     abort ();
   }
 
-#if 0
+#if defined(WIN32)
   // There's no get get_configuration function, and with some of the newer kernels
   // setting the configuration, even if to the same value, hoses any other processes
   // that have it open.  Hence we opt to not set it at all (We've only
@@ -487,18 +503,24 @@ _usrp_load_fpga (struct usb_dev_handle *udh, const char *filename,
   ok &= usrp_set_fpga_tx_enable (udh, 0);
   ok &= usrp_set_fpga_rx_enable (udh, 0);
 
-  if (_usrp_hw_rev (dev_handle_to_dev (udh)) != 0){
-    ok &= usrp_set_fpga_tx_reset (udh, 1);	// reset tx and rx  paths
-    ok &= usrp_set_fpga_rx_reset (udh, 1);
-    ok &= usrp_set_fpga_tx_reset (udh, 0);	// reset tx and rx  paths
-    ok &= usrp_set_fpga_rx_reset (udh, 0);
-  }
+  ok &= usrp_set_fpga_tx_reset (udh, 1);	// reset tx and rx paths
+  ok &= usrp_set_fpga_rx_reset (udh, 1);
+  ok &= usrp_set_fpga_tx_reset (udh, 0);	// reset tx and rx paths
+  ok &= usrp_set_fpga_rx_reset (udh, 0);
 
   if (!ok)
     fprintf (stderr, "usrp: failed to reset tx and/or rx path\n");
 
+  // Manually reset all regs except master control to zero.
+  // FIXME may want to remove this when we rework FPGA reset strategy.
+  // In the mean while, this gets us reproducible behavior.
+  for (int i = 0; i < FR_USER_0; i++){
+    if (i == FR_MASTER_CTRL)
+      continue;
+    usrp_write_fpga_reg(udh, i, 0);
+  }
+
   power_down_9862s (udh);		// on the rev1, power these down!
-  usrp_set_sleep_bits (udh, 0, 0xf);	// power up DACs and ADCs
   usrp_set_led (udh, 1, 0);		// led 1 off
 
   return true;
@@ -550,22 +572,6 @@ usrp_set_switch (struct usb_dev_handle *udh, int cmd_byte, bool on)
 }
 
 
-static bool
-usrp0_fpga_write (struct usb_dev_handle *udh, int reg, int value)
-{
-  // on the rev0 usrp, we use thd FPGA_WRITE_REG command
-  
-  unsigned char buf[4];
-
-  buf[0] = (value >> 24) & 0xff;
-  buf[1] = (value >> 16) & 0xff;
-  buf[2] = (value >>  8) & 0xff;
-  buf[3] = (value >>  0) & 0xff;
-
-  return write_cmd (udh, VRQ_FPGA_WRITE_REG,
-		    0, reg & 0xff, buf, sizeof (buf)) == sizeof (buf);
-}
-
 static bool
 usrp1_fpga_write (struct usb_dev_handle *udh,
 		  int regno, int value)
@@ -607,28 +613,24 @@ usrp1_fpga_read (struct usb_dev_handle *udh,
 bool
 usrp_write_fpga_reg (struct usb_dev_handle *udh, int reg, int value)
 {
-  switch (_usrp_hw_rev (dev_handle_to_dev (udh))){
-  case 0:
-    return usrp0_fpga_write (udh, reg, value);
-  case 1:
-  case 2:
-    return usrp1_fpga_write (udh, reg, value);
+  switch (usrp_hw_rev (dev_handle_to_dev (udh))){
+  case 0:			// not supported ;)
+    abort();	
+
   default:
-    abort ();
+    return usrp1_fpga_write (udh, reg, value);
   }
 }
 
 bool
 usrp_read_fpga_reg (struct usb_dev_handle *udh, int reg, int *value)
 {
-  switch (_usrp_hw_rev (dev_handle_to_dev (udh))){
-  case 0:
-    return false;		// no read back
-  case 1:
-  case 2:
-    return usrp1_fpga_read (udh, reg, value);
+  switch (usrp_hw_rev (dev_handle_to_dev (udh))){
+  case 0:		// not supported ;)
+    abort();
+    
   default:
-    abort ();
+    return usrp1_fpga_read (udh, reg, value);
   }
 }
 
@@ -662,18 +664,6 @@ usrp_set_fpga_rx_reset (struct usb_dev_handle *udh, bool on)
   return usrp_set_switch (udh, VRQ_FPGA_SET_RX_RESET, on);
 }
 
-bool
-usrp_set_sleep_bits (struct usb_dev_handle *udh, int bits, int mask)
-{
-  switch (_usrp_hw_rev (dev_handle_to_dev (udh))){
-  case 0:
-    bits &= 0xf;
-    mask &= 0xf;
-    return write_cmd (udh, VRQ_SET_SLEEP_BITS, (mask << 8) | bits, 0, 0, 0) == 0;
-  default:
-    return true;
-  }
-}
 
 // ----------------------------------------------------------------
 // conditional load stuff
@@ -842,32 +832,34 @@ usrp_load_firmware_nth (int nth, const char *filename, bool force){
 static void
 load_status_msg (usrp_load_status_t s, const char *type, const char *filename)
 {
+  char *e = getenv("USRP_VERBOSE");
+  bool verbose = e != 0;
+  
   switch (s){
   case ULS_ERROR:
     fprintf (stderr, "usrp: failed to load %s %s.\n", type, filename);
     break;
     
-#if (VERBOSE)
   case ULS_ALREADY_LOADED:
-    fprintf (stderr, "usrp: %s %s already loaded.\n", type, filename);
+    if (verbose)
+      fprintf (stderr, "usrp: %s %s already loaded.\n", type, filename);
     break;
 
   case ULS_OK:
-    fprintf (stderr, "usrp: %s %s loaded successfully.\n", type, filename);
+    if (verbose)
+      fprintf (stderr, "usrp: %s %s loaded successfully.\n", type, filename);
     break;
-#else
-  case ULS_ALREADY_LOADED:
-  case ULS_OK:
-    break;
-#endif
   }
 }
 
 bool
-usrp_load_standard_bits (int nth, bool force)
+usrp_load_standard_bits (int nth, bool force,
+			 const std::string fpga_filename,
+			 const std::string firmware_filename)
 {
   usrp_load_status_t 	s;
-  char			*filename;
+  const char		*filename;
+  const char		*proto_filename;
   int hw_rev;
 
   // first, figure out what hardware rev we're dealing with
@@ -877,14 +869,16 @@ usrp_load_standard_bits (int nth, bool force)
       fprintf (stderr, "usrp: failed to find usrp[%d]\n", nth);
       return false;
     }
-    hw_rev = _usrp_hw_rev (udev);
+    hw_rev = usrp_hw_rev (udev);
   }
 
   // start by loading the firmware
 
-  filename = find_file (usrp_firmware_filename, hw_rev);
+  proto_filename = get_proto_filename(firmware_filename, "USRP_FIRMWARE",
+				      default_firmware_filename);
+  filename = find_file(proto_filename, hw_rev);
   if (filename == 0){
-    fprintf (stderr, "Can't find firmware: %s\n", usrp_firmware_filename);
+    fprintf (stderr, "Can't find firmware: %s\n", proto_filename);
     return false;
   }
 
@@ -900,9 +894,11 @@ usrp_load_standard_bits (int nth, bool force)
 
   // now move on to the fpga configuration bitstream
 
-  filename = find_file (usrp_fpga_filename, hw_rev);
+  proto_filename = get_proto_filename(fpga_filename, "USRP_FPGA",
+				      default_fpga_filename);
+  filename = find_file (proto_filename, hw_rev);
   if (filename == 0){
-    fprintf (stderr, "Can't find fpga bitstream: %s\n", usrp_fpga_filename);
+    fprintf (stderr, "Can't find fpga bitstream: %s\n", proto_filename);
     return false;
   }
 
@@ -1069,9 +1065,11 @@ power_down_9862s (struct usb_dev_handle *udh)
     REG_TX_MODULATOR,	0x00			// coarse & fine modulators disabled
   };
 
-  switch (_usrp_hw_rev (dev_handle_to_dev (udh))){
-  case 1:
-  case 2:
+  switch (usrp_hw_rev (dev_handle_to_dev (udh))){
+  case 0:
+    break;
+
+  default:
     usrp_9862_write_many_all (udh, regs, sizeof (regs));
     break;
   }
@@ -1341,3 +1339,17 @@ usrp_write_dboard_offsets (struct usb_dev_handle *udh, int slot_id,
   return usrp_eeprom_write (udh, slot_to_i2c_addr (slot_id),
 			    0, buf, sizeof (buf));
 }
+
+std::string
+usrp_serial_number(struct usb_dev_handle *udh)
+{
+  u_int8_t iserial = usb_device(udh)->descriptor.iSerialNumber;
+  if (iserial == 0)
+    return "";
+
+  char buf[1024];
+  if (usb_get_string_simple(udh, iserial, buf, sizeof(buf)) < 0)
+    return "";
+
+  return buf;
+}
